from typing import Any
from uuid import uuid4

from pydantic import BaseModel
from pydantic import Field


class LockObject(BaseModel):
    """
    Represents a lock object with an ID, expiration time, and associated data.

    Attributes:
        lock_id (str): The unique identifier for the lock, generated by default.
        expires_at (int): The timestamp when the lock expires.
        data (dict[str, Any]): Additional data associated with the lock.
    """

    lock_id: str = Field(default_factory=lambda: uuid4().hex)
    expires_at: int
    data: dict[str, Any]
